@@ -25,37 +25,41 @@
 #include "gnome-vfs-mime-handlers.h"
 
 #include "gnome-vfs-mime-info.h"
-#ifdef HAVE_GCONF
+#include "gnome-vfs-application-registry.h"
 #include <gconf/gconf.h>
-#endif
-#include <gtk/gtksignal.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#include <libgnome/gnome-util.h>
 #include <sys/stat.h>
 #include "gnome-vfs-result.h"
 
-static char *get_user_level (void);
-static gboolean str_to_bool (const char *str);
-static const char *bool_to_str (gboolean bool);
-static char *extract_prefix_add_suffix (const char *string, const char *separator, const char *suffix);
-static char *mime_type_get_supertype (const char *mime_type);
-static char **strsplit_handle_null (const char *str, const char *delim, int max);
-#ifdef USING_OAF
-static char *join_str_list (const char *separator, GList *list);
-static GList *OAF_ServerInfoList_to_ServerInfo_g_list (OAF_ServerInfoList *info_list);
-#endif
-static GList *process_app_list (GList *id_list);
-static GList *comma_separated_str_to_str_list (const char *str);
-static GList *str_list_difference (GList *a, GList *b);
-static char *str_list_to_comma_separated_str (GList *list);
-static GList *gnome_vfs_strsplit_to_list (const char *str, const char *delim, int max);
-static char *gnome_vfs_strjoin_from_list (const char *separator, GList *list);
-static void g_list_free_deep (GList *list);
-static GList *prune_ids_for_nonexistent_applications (GList *list);
-static GnomeVFSResult gnome_vfs_mime_edit_user_file (const char *mime_type, const char *key, const char *value);
+static char *         get_user_level                          (void);
+static char *         extract_prefix_add_suffix               (const char         *string,
+							       const char         *separator,
+							       const char         *suffix);
+static char *         mime_type_get_supertype                 (const char         *mime_type);
+static char **        strsplit_handle_null                    (const char         *str,
+							       const char         *delim,
+							       int                 max);
+static char *         join_str_list                           (const char         *separator,
+							       GList              *list);
+static GList *        OAF_ServerInfoList_to_ServerInfo_g_list (OAF_ServerInfoList *info_list);
+static GList *        comma_separated_str_to_str_list         (const char         *str);
+static GList *        str_list_difference                     (GList              *a,
+							       GList              *b);
+static char *         str_list_to_comma_separated_str         (GList              *list);
+static GList *        gnome_vfs_strsplit_to_list              (const char         *str,
+							       const char         *delim,
+							       int                 max);
+static char *         gnome_vfs_strjoin_from_list             (const char         *separator,
+							       GList              *list);
+static void           g_list_free_deep                        (GList              *list);
+static GList *        prune_ids_for_nonexistent_applications  (GList              *list);
+static GnomeVFSResult gnome_vfs_mime_edit_user_file           (const char         *mime_type,
+							       const char         *key,
+							       const char         *value);
 
+static GConfEngine *gconf_engine = NULL;
 
 static GnomeVFSMimeActionType
 gnome_vfs_mime_get_default_action_type_without_fallback (const char *mime_type)
@@ -74,6 +78,22 @@ gnome_vfs_mime_get_default_action_type_without_fallback (const char *mime_type)
 	}
 }
 
+
+
+/**
+ * gnome_vfs_mime_get_description:
+ * @mime_type: the mime type
+ *
+ * Returns the description for this mime-type
+ */
+const char *
+gnome_vfs_mime_get_description (const char *mime_type)
+{
+	return gnome_vfs_mime_get_value (mime_type, "description");
+}
+
+
+
 GnomeVFSMimeActionType
 gnome_vfs_mime_get_default_action_type (const char *mime_type)
 {
@@ -82,6 +102,14 @@ gnome_vfs_mime_get_default_action_type (const char *mime_type)
 
 	action_type = gnome_vfs_mime_get_default_action_type_without_fallback (mime_type);
 
+	/* FIXME bugzilla.eazel.com 2650: the current (and the previous) implementation of 
+	   gnome_vfs_mime_get_value
+	   tries the supertype. This means that the folowing code is useless.
+	   The correct fix for this is to add a new funciotn in gnome-vfs-mime-info.c
+	   gnome_vfs_mime_get_value_without_callback. Waiting for evolution guys to fix it.
+	   Might remove this code too if we can convince them that wht they do with this 
+	   function is bad.
+	*/
 	if (action_type == GNOME_VFS_MIME_ACTION_TYPE_NONE) {
 		/* Fall back to the supertype. */
 		supertype = mime_type_get_supertype (mime_type);
@@ -112,16 +140,12 @@ gnome_vfs_mime_get_default_action_without_fallback (const char *mime_type)
 		}
 		break;
 	case GNOME_VFS_MIME_ACTION_TYPE_COMPONENT:
-#ifdef USING_OAF
 		action->action.component = 
 			gnome_vfs_mime_get_default_component (mime_type);
 		if (action->action.component == NULL) {
 			g_free (action);
 			action = NULL;
 		}
-#else
-		action= NULL;
-#endif
 		break;
 	case GNOME_VFS_MIME_ACTION_TYPE_NONE:
 		g_free (action);
@@ -169,14 +193,16 @@ gnome_vfs_mime_get_default_application (const char *mime_type)
 			default_application = gnome_vfs_mime_get_default_application (supertype);
 			g_free (supertype);
 			return default_application;
+		} else {
+			return NULL;
 		}
 	}
 
-	/* FIXME: Instead of returning NULL we need to chose from
+	/* FIXME bugzilla.eazel.com 2756: Instead of returning NULL we need to chose from
 	 * pruned system lists here.
 	 */
 
-	return gnome_vfs_mime_application_new_from_id (default_application_id);
+	return gnome_vfs_application_registry_get_mime_application (default_application_id);
 }
 
 const char *
@@ -193,7 +219,6 @@ gnome_vfs_mime_set_icon (const char *mime_type, const char *filename)
 }
 
 
-#ifdef USING_OAF
 OAF_ServerInfo *
 gnome_vfs_mime_get_default_component (const char *mime_type)
 {
@@ -251,11 +276,11 @@ gnome_vfs_mime_get_default_component (const char *mime_type)
 
 	short_list = gnome_vfs_mime_get_short_list_components (mime_type);
 
-	if (p != NULL) {
+	if (short_list != NULL) {
 		sort[1] = g_strdup ("has (['");
 
 		for (p = short_list; p != NULL; p = p->next) {
-			prev = sort[1];
+ 			prev = sort[1];
 			
 			if (p->next != NULL) {
 				sort[1] = g_strconcat (prev, ((OAF_ServerInfo *) (p->data))->iid, 
@@ -305,7 +330,6 @@ gnome_vfs_mime_get_default_component (const char *mime_type)
 
 	return default_component;
 }
-#endif
 
 static GList *
 gnome_vfs_mime_str_list_merge (GList *a, 
@@ -385,7 +409,7 @@ static GList *gnome_vfs_mime_do_short_list_processing (GList *short_list,
 									supertype_removals);
 
 	merged_system_and_supertype = gnome_vfs_mime_str_list_merge (short_list,
-								     processed_supertype_list);
+					     			     processed_supertype_list);
 
 	final_list = gnome_vfs_mime_str_list_apply_delta (merged_system_and_supertype,
 							  additions,
@@ -398,6 +422,28 @@ static GList *gnome_vfs_mime_do_short_list_processing (GList *short_list,
 }
 
 
+/* sort_application_list
+ *
+ * Sort list alphabetically
+ */
+ 
+static int
+sort_application_list (gconstpointer a, gconstpointer b)
+{
+	GnomeVFSMimeApplication *application1, *application2;
+
+	application1 = (GnomeVFSMimeApplication *) a;
+	application2 = (GnomeVFSMimeApplication *) b;
+
+	return g_strcasecmp (application1->name, application2->name);
+}
+
+/* gnome_vfs_mime_get_short_list_applications
+ *
+ * Return alphabetically sorted list of GnomeVFSMimeApplication
+ * data structures for the requested mime type.	
+ */
+ 
 GList *
 gnome_vfs_mime_get_short_list_applications (const char *mime_type)
 {
@@ -433,11 +479,11 @@ gnome_vfs_mime_get_short_list_applications (const char *mime_type)
 							       (mime_type,
 								"short_list_application_user_removals"));
 
-	/* Get supertype info to merge in. */
-
+	/* Only include the supertype in the short list if we came up empty with
+	   the specific types */
 	supertype = mime_type_get_supertype (mime_type);
 
-	if (strcmp (supertype, mime_type) != 0) {
+	if ((strcmp (supertype, mime_type) != 0) && (system_short_list == NULL)) {
 		supertype_short_list = comma_separated_str_to_str_list 
 			(gnome_vfs_mime_get_value_for_user_level 
 			 (supertype, 
@@ -474,7 +520,7 @@ gnome_vfs_mime_get_short_list_applications (const char *mime_type)
 	preferred_applications = NULL;
 
 	for (p = id_list; p != NULL; p = p->next) {
-		application = gnome_vfs_mime_application_new_from_id (p->data);
+		application = gnome_vfs_application_registry_get_mime_application (p->data);
 		if (application != NULL) {
 			preferred_applications = g_list_prepend
 				(preferred_applications, application);
@@ -492,11 +538,13 @@ gnome_vfs_mime_get_short_list_applications (const char *mime_type)
 	g_list_free_deep (supertype_removals);
 	g_list_free (id_list);
 
+	/* Sort list alphabetically by application name */
+	preferred_applications = g_list_sort (preferred_applications, sort_application_list);
+	
 	return preferred_applications;
 }
 
 
-#ifdef USING_OAF
 static char *
 join_str_list (const char *separator, GList *list)
 {
@@ -522,12 +570,27 @@ join_str_list (const char *separator, GList *list)
 
 	return retval;
 }
-#endif
+
+
+/* sort_component_list
+ *
+ * Sort list alphabetically by component name
+ */
+ 
+static int
+sort_component_list (gconstpointer a, gconstpointer b)
+{
+	OAF_ServerInfo *component1, *component2;
+		
+	component1 = (OAF_ServerInfo *) a;
+	component2 = (OAF_ServerInfo *) b;
+
+	return g_strcasecmp (component1->iid, component2->iid);
+}
 
 GList *
 gnome_vfs_mime_get_short_list_components (const char *mime_type)
 {
-#ifdef USING_OAF
 	GList *system_short_list;
 	GList *short_list_additions;
 	GList *short_list_removals;
@@ -607,7 +670,7 @@ gnome_vfs_mime_get_short_list_components (const char *mime_type)
 		CORBA_exception_init (&ev);
 
 		iids_delimited = join_str_list ("','", iid_list);
-		
+
 		/* FIXME bugzilla.eazel.com 1142: should probably check for
 		   the right interfaces too. Also slightly semantically
 		   different from nautilus in other tiny ways.
@@ -644,10 +707,10 @@ gnome_vfs_mime_get_short_list_components (const char *mime_type)
 	g_list_free_deep (supertype_removals);
 	g_list_free (iid_list);
 
+	/* Sort list alphabetically by component name */
+	preferred_components = g_list_sort (preferred_components, sort_component_list);
+
 	return preferred_components;
-#else
-	return NULL;
-#endif
 }
 
 
@@ -655,82 +718,36 @@ gnome_vfs_mime_get_short_list_components (const char *mime_type)
 GList *
 gnome_vfs_mime_get_all_applications (const char *mime_type)
 {
- 	GList *system_all_application_ids;
- 	GList *user_all_application_ids;
-	char *supertype;
- 	GList *supertype_all_application_ids;
- 	GList *supertype_user_all_application_ids;
-	GList *full_normal;
-	GList *full_supertype;
-	GList *full_id_list;
-	GList *system_apps;
+	GList *applications, *li;
 
-	if (mime_type == NULL) {
-		return NULL;
+	g_return_val_if_fail (mime_type != NULL, NULL);
+
+	applications = gnome_vfs_application_registry_get_applications (mime_type);
+
+	/* convert list to list of GnomeVFSMimeApplication's */
+	li = applications;
+	while (li != NULL) {
+		const char *application_id = li->data;
+		li->data = gnome_vfs_application_registry_get_mime_application (application_id);
+		/* Make sure we get no NULLs */
+		if (li->data == NULL) {
+			GList *link_to_remove = li;
+
+			li = li->next;
+
+			applications = g_list_remove_link (applications,
+							   link_to_remove);
+			g_list_free_1 (link_to_remove);
+		} else
+			li = li->next;
 	}
 
-	/* FIXME bugzilla.eazel.com 1075: 
-	   no way for apps to modify at install time */
-
-	/* get app list */
- 	system_all_application_ids = comma_separated_str_to_str_list 
-		(gnome_vfs_mime_get_value 
-		 (mime_type, "system_all_application_ids"));
-	system_all_application_ids = prune_ids_for_nonexistent_applications
-		(system_all_application_ids);
-
- 	user_all_application_ids = comma_separated_str_to_str_list
-		(gnome_vfs_mime_get_value 
-		 (mime_type, "all_application_ids"));
-
-	supertype = mime_type_get_supertype (mime_type);
-	
-	if (strcmp (supertype, mime_type) != 0) {
-		supertype_all_application_ids = comma_separated_str_to_str_list 
-			(gnome_vfs_mime_get_value
-			 (supertype, 
-			  "system_all_application_ids"));
-		supertype_all_application_ids = prune_ids_for_nonexistent_applications
-			(supertype_all_application_ids);
-
-		supertype_user_all_application_ids = comma_separated_str_to_str_list 
-			(gnome_vfs_mime_get_value
-			 (supertype, 
-			  "all_application_ids"));
-
-	} else {
-		supertype_all_application_ids = NULL;
-		supertype_user_all_application_ids = NULL;
-	}
-
-	full_normal = gnome_vfs_mime_str_list_merge (system_all_application_ids,
-						     user_all_application_ids); 
-		
-	full_supertype = gnome_vfs_mime_str_list_merge (supertype_all_application_ids,
-							supertype_user_all_application_ids); 
-
-	full_id_list = gnome_vfs_mime_str_list_merge (full_normal, full_supertype);
-
-
-	
-	system_apps = process_app_list (full_id_list);
-	
-	g_free (supertype);
-	g_list_free_deep (system_all_application_ids);
-	g_list_free_deep (user_all_application_ids);
-	g_list_free_deep (supertype_all_application_ids);
-	g_list_free_deep (supertype_user_all_application_ids);
-	g_list_free (full_normal);
-	g_list_free (full_supertype);
-	g_list_free (full_id_list);
-
-	return system_apps;
+	return applications;
 }
 
 GList *
 gnome_vfs_mime_get_all_components (const char *mime_type)
 {
-#ifdef USING_OAF
 	OAF_ServerInfoList *info_list;
 	GList *components_list;
 	CORBA_Environment ev;
@@ -776,17 +793,12 @@ gnome_vfs_mime_get_all_components (const char *mime_type)
 	CORBA_exception_free (&ev);
 
 	return components_list;
-#else
-	return NULL;
-#endif
 }
 
 static GnomeVFSResult
 gnome_vfs_mime_edit_user_file_full (const char *mime_type, GList *keys, GList *values)
 {
 	GnomeVFSResult result;
-	char *user_mime_file;
-	FILE *f;
 	GList *p, *q;
 	const char *key, *value;
 
@@ -794,38 +806,18 @@ gnome_vfs_mime_edit_user_file_full (const char *mime_type, GList *keys, GList *v
 		return GNOME_VFS_OK;
 	}
 
-	user_mime_file = gnome_util_home_file ("mime-info/user.keys");
-
-	/* FIXME bugzilla.eazel.com 1119: Is it OK to always append? */
 	result = GNOME_VFS_OK;
-	f = fopen (user_mime_file, "a");
-	if (f == NULL) {
-		result = gnome_vfs_result_from_errno ();
-	}
-	if (result == GNOME_VFS_OK && fprintf (f, "\n%s:\n", mime_type) <= 0) {
-		result = gnome_vfs_result_from_errno ();
-	}
-	if (result == GNOME_VFS_OK) {
-		for (p = keys, q = values; p != NULL && q != NULL; p = p->next, q = q->next) {
-			key = p->data;
-			value = q->data;
-			if (value == NULL) {
-				value = "";
-			}
 
-			if (fprintf (f, "\t%s=%s\n", key, value) <= 0) {
-				result = gnome_vfs_result_from_errno ();
-				break;
-			}
+	gnome_vfs_mime_freeze ();
+	for (p = keys, q = values; p != NULL && q != NULL; p = p->next, q = q->next) {
+		key = p->data;
+		value = q->data;
+		if (value == NULL) {
+			value = "";
 		}
+		gnome_vfs_mime_set_value (mime_type, g_strdup (key), g_strdup (value));
 	}
-	if (result == GNOME_VFS_OK && fclose (f) != 0) {
-		result = gnome_vfs_result_from_errno ();
-	}
-
-	g_free (user_mime_file);
-
-	gnome_vfs_mime_info_reload ();
+	gnome_vfs_mime_thaw ();
 
 	return result;
 }
@@ -1097,13 +1089,10 @@ gnome_vfs_mime_id_in_application_list (const char *id, GList *applications)
 gboolean
 gnome_vfs_mime_id_in_component_list (const char *iid, GList *components)
 {
-#ifdef USING_OAF
 	return g_list_find_custom
 		(components, (gpointer) iid,
 		 (GCompareFunc) gnome_vfs_mime_component_matches_id) != NULL;
-#else
 	return FALSE;
-#endif
 }
 
 /**
@@ -1150,15 +1139,12 @@ GList *
 gnome_vfs_mime_id_list_from_component_list (GList *components)
 {
 	GList *list = NULL;
-#ifdef USING_OAF
 	GList *node;
 
 	for (node = components; node != NULL; node = node->next) {
 		list = g_list_prepend 
 			(list, g_strdup (((OAF_ServerInfo *)node->data)->iid));
 	}
-#endif
-
 	return g_list_reverse (list);
 }
 
@@ -1295,7 +1281,6 @@ gnome_vfs_mime_remove_component_from_list (GList *components,
 					   const char *iid,
 					   gboolean *did_remove)
 {
-#ifdef USING_OAF
 	GList *matching_node;
 	
 	matching_node = g_list_find_custom 
@@ -1309,8 +1294,6 @@ gnome_vfs_mime_remove_component_from_list (GList *components,
 	if (did_remove != NULL) {
 		*did_remove = matching_node != NULL;
 	}
-#endif
-
 	return components;
 }
 
@@ -1365,6 +1348,10 @@ gnome_vfs_mime_add_extension (const char *mime_type, const char *extension)
 	
 	list = gnome_vfs_mime_get_extensions_list (mime_type);	
 	if (list == NULL) {
+		/* List is NULL. This means there are no current registered extensions. 
+		 * Add the new extension and it will cause the list to be created next time.
+		 */
+		result = gnome_vfs_mime_set_registered_type_key (mime_type, "ext", extension);
 		return result;
 	}
 
@@ -1394,7 +1381,6 @@ gnome_vfs_mime_add_extension (const char *mime_type, const char *extension)
 
 		/* Add extensions to hash table and flush into the file. */
 		gnome_vfs_mime_set_registered_type_key (mime_type, "ext", extensions);
-		result = gnome_vfs_mime_commit_registered_types ();
 	}
 	
 	gnome_vfs_mime_extensions_list_free (list);
@@ -1455,7 +1441,6 @@ gnome_vfs_mime_remove_extension (const char *mime_type, const char *extension)
 	if (extensions != NULL) {
 		/* Add extensions to hash table and flush into the file */
 		gnome_vfs_mime_set_registered_type_key (mime_type, "ext", extensions);
-		result = gnome_vfs_mime_commit_registered_types ();
 	}
 	
 	gnome_vfs_mime_extensions_list_free (list);
@@ -1468,75 +1453,34 @@ GnomeVFSResult
 gnome_vfs_mime_extend_all_applications (const char *mime_type,
 					GList *application_ids)
 {
-	const char *user_all_application_ids;
-	GList *user_id_list;
-	GList *extras;
-	GList *update_list;
-	char *update_str;
-	GnomeVFSResult result;
+	GList *li;
 
-	user_all_application_ids = gnome_vfs_mime_get_value 
-		(mime_type, "all_application_ids");
-	user_id_list = comma_separated_str_to_str_list (user_all_application_ids);
+	g_return_val_if_fail (mime_type != NULL, GNOME_VFS_ERROR_INTERNAL);
 
-	extras = str_list_difference (application_ids, user_id_list);
+	for (li = application_ids; li != NULL; li = li->next) {
+		const char *application_id = li->data;
+		gnome_vfs_application_registry_add_mime_type (application_id,
+							      mime_type);
+	}
 
-	update_list = g_list_concat (g_list_copy (user_id_list), extras);
-
-	update_str = str_list_to_comma_separated_str (update_list);
-
-	g_list_free_deep (user_id_list);
-	
-	result = gnome_vfs_mime_edit_user_file
-		(mime_type, "all_application_ids", update_str);
-	g_free (update_str);
-
-	return result;
+	return gnome_vfs_application_registry_sync ();
 }
 
 GnomeVFSResult
 gnome_vfs_mime_remove_from_all_applications (const char *mime_type,
 					     GList *application_ids)
 {
-	GnomeVFSResult result;
-	const char *user_all_application_ids;
-	GList *user_id_list;
-	GList *update_list;
-	char *update_str;
+	GList *li;
 
- 	user_all_application_ids = gnome_vfs_mime_get_value 
-		(mime_type, "all_application_ids");
-	user_id_list = comma_separated_str_to_str_list (user_all_application_ids);
+	g_return_val_if_fail (mime_type != NULL, GNOME_VFS_ERROR_INTERNAL);
 
-	update_list = str_list_difference (user_id_list, application_ids);
-	update_str = str_list_to_comma_separated_str (update_list);
+	for (li = application_ids; li != NULL; li = li->next) {
+		const char *application_id = li->data;
+		gnome_vfs_application_registry_remove_mime_type (application_id,
+								 mime_type);
+	}
 
-	g_list_free_deep (user_id_list);
-	
-	result = gnome_vfs_mime_edit_user_file
-		(mime_type, "all_application_ids", update_str);
-	g_free (update_str);
-
-	return result;
-}
-
-GnomeVFSResult
-gnome_vfs_mime_define_application (const char *mime_type, GnomeVFSMimeApplication *application)
-{
-	GnomeVFSResult result;
-
-	g_return_val_if_fail (application != NULL, GNOME_VFS_OK);
-	g_return_val_if_fail (mime_type != NULL, GNOME_VFS_OK);
-
-	result = gnome_vfs_mime_edit_user_file_multiple
-		(mime_type,
-		 "name", application->name,
-		 "command", application->command,
-		 "can_open_multiple_files", bool_to_str (application->can_open_multiple_files),
-		 "can_open_uris", bool_to_str (application->can_open_uris),
-		 NULL);
-
-	return result;
+	return gnome_vfs_application_registry_sync ();
 }
 
 GnomeVFSMimeApplication *
@@ -1578,9 +1522,7 @@ gnome_vfs_mime_action_free (GnomeVFSMimeAction *action)
 		gnome_vfs_mime_application_free (action->action.application);
 		break;
 	case GNOME_VFS_MIME_ACTION_TYPE_COMPONENT:
-#ifdef USING_OAF
 		CORBA_free (action->action.component);
-#endif
 		break;
 	default:
 		g_assert_not_reached ();
@@ -1599,28 +1541,10 @@ gnome_vfs_mime_application_list_free (GList *list)
 void
 gnome_vfs_mime_component_list_free (GList *list)
 {
-#ifdef USING_OAF
 	g_list_foreach (list, (GFunc) CORBA_free, NULL);
-#endif
 	g_list_free (list);
 }
 
-
-
-static gboolean
-str_to_bool (const char *str)
-{
-	return str != NULL &&
-		(strcasecmp (str, "true") == 0 ||
-		 strcasecmp (str, "yes") == 0);
-}
-
-static const char *
-bool_to_str (gboolean bool)
-{
-	return bool ? "true" : "false";
-}
-
 static char *
 extract_prefix_add_suffix (const char *string, const char *separator, const char *suffix)
 {
@@ -1663,48 +1587,8 @@ GnomeVFSMimeApplication *
 gnome_vfs_mime_application_new_from_id (const char *id)
 {
 	GnomeVFSMimeApplication *application;
-	const char *command;
-	const char *name;
-	char *id_hack_mime_type;
 
-	if (id == NULL) {
-		return NULL;
-	}
-
-	application = g_new0 (GnomeVFSMimeApplication, 1);
-
-	id_hack_mime_type = g_strconcat ("x-application-registry-hack/", id, NULL);
-
-	command = gnome_vfs_mime_get_value (id_hack_mime_type, 
-					    "command");
-
-	if (command == NULL) {
-		g_free (id_hack_mime_type);
-		g_free (application);
-		return NULL;
-	} else {
-		application->command = g_strdup (command);
-	}
-
-	name = gnome_vfs_mime_get_value (id_hack_mime_type, 
-					 "name");
-
-	if (name == NULL) {
-		application->name = gnome_vfs_mime_program_name (application->command);
-	} else {
-		application->name = g_strdup (name);
-	}
-
-	application->can_open_multiple_files = str_to_bool (gnome_vfs_mime_get_value (id_hack_mime_type, 
-										      "can_open_multiple_files"));
-	
-	application->can_open_uris = str_to_bool (gnome_vfs_mime_get_value (id_hack_mime_type, 
-									    "can_open_uris"));
-
-	application->requires_terminal = str_to_bool (gnome_vfs_mime_get_value (id_hack_mime_type, 
-										"requires_terminal"));
-
-	application->id = g_strdup (id);
+	application = gnome_vfs_application_registry_get_mime_application (id);
 
 	return application;
 }
@@ -1780,23 +1664,23 @@ executable_in_path (const char *executable_name)
 static char *
 get_executable_name_from_command_string (const char *command)
 {
-	/* FIXME: Do we need to handle quoting? */
+	/* FIXME bugzilla.eazel.com 2757: Do we need to handle quoting? */
 	return strdup_to (command, strchr (command, ' '));
 }
 
 static gboolean
 application_known_to_be_nonexistent (const char *application_id)
 {
-	char *id_hack_mime_type;
 	const char *command;
 	char *executable_name;
 	gboolean found;
 
-	id_hack_mime_type = g_strconcat ("x-application-registry-hack/",
-					 application_id,
-					 NULL);
-	command = gnome_vfs_mime_get_value (id_hack_mime_type,
-					    "command");
+	g_return_val_if_fail (application_id != NULL, FALSE);
+
+	command = gnome_vfs_application_registry_peek_value
+		(application_id,
+		 GNOME_VFS_APPLICATION_REGISTRY_COMMAND);
+
 	if (command == NULL) {
 		return TRUE;
 	}
@@ -1827,7 +1711,6 @@ prune_ids_for_nonexistent_applications (GList *list)
 	return list;
 }
 
-#ifdef USING_OAF
 static GList *
 OAF_ServerInfoList_to_ServerInfo_g_list (OAF_ServerInfoList *info_list)
 {
@@ -1844,32 +1727,13 @@ OAF_ServerInfoList_to_ServerInfo_g_list (OAF_ServerInfoList *info_list)
 
 	return retval;
 }
-#endif
 
-static GList *
-process_app_list (GList *id_list) 
+static void
+unref_gconf_engine (void)
 {
-	GList *applications;
-	GList *p;
-
-	applications = NULL;
-
-	for (p = id_list; p != NULL; p = p->next) {
-		GnomeVFSMimeApplication *application;
-
-		application = gnome_vfs_mime_application_new_from_id (p->data);
-		
-		if (application != NULL) {
-			applications = g_list_prepend (applications, application);
-		}
-	}
-
-	applications = g_list_reverse (applications);
-
-	return applications;
+	gconf_engine_unref (gconf_engine);
 }
 
-
 /* Returns the Nautilus user level, a string.
  * This does beg the question: Why does gnome-vfs have the Nautilus
  * user level coded into it? Eventually we might want to call this the
@@ -1880,23 +1744,20 @@ static char *
 get_user_level (void)
 {
 	char *user_level = NULL;
-#ifdef HAVE_GCONF
-	static GConfEngine *engine = NULL;
 
 	/* Create the gconf engine once. */
-	if (engine == NULL) {
+	if (gconf_engine == NULL) {
 		/* This sequence is needed in case no one has initialized GConf. */
 		if (!gconf_is_initialized ()) {
 			char *fake_argv[] = { "gnome-vfs", NULL };
 			gconf_init (1, fake_argv, NULL);
 		}
 
-		engine = gconf_engine_new ();
-		/* FIXME bugzilla.eazel.com 1150: This engine never gets freed. */
+		gconf_engine = gconf_engine_get_default ();
+		g_atexit (unref_gconf_engine);
 	}
 
-	user_level = gconf_get_string (engine, "/apps/nautilus/user_level", NULL);
-#endif
+	user_level = gconf_engine_get_string (gconf_engine, "/apps/nautilus/user_level", NULL);
 
 	if (user_level == NULL) {
 		user_level = g_strdup ("novice");
