@@ -1,5 +1,5 @@
 /*
- * gnome-vfs-mime-magic-magic.c
+ * gnome-vfs-mime-magic.c
  *
  * Written by:
  *    James Youngman (jay@gnu.org)
@@ -23,7 +23,6 @@
 #include "gnome-vfs-mime-sniff-buffer-private.h"
 #include "gnome-vfs-mime.h"
 
-#include <libgnome/libgnome.h>
 #include <ctype.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -53,7 +52,7 @@ is_hex_digit (char ch)
 	return (ch >= 'A' && ch <= 'F');
 }
 
-/* FIXME:
+/* FIXME bugzilla.eazel.com 2760:
  * should return error here
  */
 static guchar
@@ -76,7 +75,7 @@ read_octal_byte (const char **pos)
 	return retval;
 }
 
-/* FIXME:
+/* FIXME bugzilla.eazel.com 2760:
  * should return error here
  */
 static guchar
@@ -106,7 +105,7 @@ read_hex_byte (const char **pos)
 	return retval;
 }
 
-/* FIXME:
+/* FIXME bugzilla.eazel.com 2760:
  * should return error here
  */
 static const char *
@@ -377,7 +376,7 @@ gnome_vfs_mime_magic_parse (const gchar *filename, gint *nents)
 
 		if (newent.type == T_STR) {
 			scanner = read_string_val (scanner, newent.pattern, 
-				sizeof (newent.pattern), &newent.pattern_length);
+						   sizeof (newent.pattern), &newent.pattern_length);
 		} else {
 			newent.pattern_length = bsize;
 			if (!read_num_val (&scanner, bsize, (int *)&newent.pattern)) {
@@ -448,11 +447,11 @@ endian_swap (guchar *result, const guchar *data, size_t length)
 }
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
- #define FIRST_ENDIAN_DEPENDENT_TYPE T_BESHORT
- #define LAST_ENDIAN_DEPENDENT_TYPE T_BEDATE
+#define FIRST_ENDIAN_DEPENDENT_TYPE T_BESHORT
+#define LAST_ENDIAN_DEPENDENT_TYPE T_BEDATE
 #else	
- #define FIRST_ENDIAN_DEPENDENT_TYPE T_LESHORT
- #define LAST_ENDIAN_DEPENDENT_TYPE T_LEDATE
+#define FIRST_ENDIAN_DEPENDENT_TYPE T_LESHORT
+#define LAST_ENDIAN_DEPENDENT_TYPE T_LEDATE
 #endif
 
 static gboolean
@@ -460,7 +459,8 @@ try_one_pattern_on_buffer (const char *sniffed_stream, GnomeMagicEntry *magic_en
 {
 	gboolean using_cloned_pattern;
 	char pattern_clone [48];
-	int index;
+	int index, count;
+	const char *pattern;
 
 	using_cloned_pattern = FALSE;
 	if (magic_entry->type >= FIRST_ENDIAN_DEPENDENT_TYPE && magic_entry->type <= LAST_ENDIAN_DEPENDENT_TYPE) { 
@@ -478,7 +478,7 @@ try_one_pattern_on_buffer (const char *sniffed_stream, GnomeMagicEntry *magic_en
 		using_cloned_pattern = TRUE;
 	}
 
-	if (magic_entry->use_mask ) {
+	if (magic_entry->use_mask) {
 		/* Apply mask to the examined data. At this point the data in
 		 * sniffed_stream is in the same endianness as the mask.
 		 */ 
@@ -494,21 +494,49 @@ try_one_pattern_on_buffer (const char *sniffed_stream, GnomeMagicEntry *magic_en
 		}
 	}
 
-	return memcmp (magic_entry->pattern, sniffed_stream, magic_entry->pattern_length) == 0;
+	if (*magic_entry->pattern != *sniffed_stream) {
+		return FALSE;
+	}
+	
+	for (count = magic_entry->pattern_length, pattern = magic_entry->pattern;
+	     count > 0; count--) {
+		if (*pattern++ != *sniffed_stream++) {
+			return FALSE;
+		}
+	}
+	return TRUE;
 }
 
+enum {
+	SNIFF_BUFFER_CHUNK = 32
+};
+
 
 static gboolean
 gnome_vfs_mime_try_one_magic_pattern (GnomeVFSMimeSniffBuffer *sniff_buffer, 
-	GnomeMagicEntry *magic_entry)
+				      GnomeMagicEntry *magic_entry)
 {
 	int offset;
 
+	if (sniff_buffer->read_whole_file &&
+	    sniff_buffer->buffer_length < magic_entry->range_start + magic_entry->pattern_length) {
+		/* There's no place this pattern could actually match */
+		return FALSE;
+	}
 	for (offset = magic_entry->range_start; offset <= magic_entry->range_end; offset++) {
-		if (gnome_vfs_mime_sniff_buffer_get (sniff_buffer, 
-			offset + magic_entry->pattern_length) != GNOME_VFS_OK) {
-			return FALSE;
+		/* this check is done only as an optimization
+		 * gnome_vfs_mime_sniff_buffer_get already implements the laziness.
+		 * This gets called a million times though and every bit performance
+		 * is valuable. This way we avoid making the call.
+		 */
+		if (sniff_buffer->buffer_length < offset + magic_entry->pattern_length &&
+		    !sniff_buffer->read_whole_file) {
+			if (gnome_vfs_mime_sniff_buffer_get (sniff_buffer, 
+							     offset + magic_entry->pattern_length) != GNOME_VFS_OK) {
+				return FALSE;
+			}
 		}
+		
 		if (try_one_pattern_on_buffer (sniff_buffer->buffer + offset, magic_entry)) {
 			return TRUE;
 		}
@@ -521,7 +549,7 @@ G_LOCK_DEFINE_STATIC (mime_magic_table_mutex);
 
 static GnomeMagicEntry *mime_magic_table = NULL;
 
-/* FIXME: This whole mmap optimization may be unnecessary. It complicates
+/* FIXME bugzilla.eazel.com 2761: This whole mmap optimization may be unnecessary. It complicates
  * this code quite a bit (and slows down "make install"). If it's not buying
  * a noticeable performance improvement we should get rid of it.
  */
@@ -553,10 +581,10 @@ gnome_vfs_mime_get_magic_table (void)
 			if (file >= 0) {
 				if (fstat (file, &sbuf) == 0) {
 					mmap_result = (GnomeMagicEntry *) mmap(NULL, 
-						sbuf.st_size, 
-						PROT_READ, 
-						MAP_SHARED, 
-						file, 0);
+									       sbuf.st_size, 
+									       PROT_READ, 
+									       MAP_SHARED, 
+									       file, 0);
 					if (mmap_result != MAP_FAILED) {
 						mime_magic_table_size = sbuf.st_size;
 						mime_magic_table_is_mapped = TRUE;
@@ -574,6 +602,7 @@ gnome_vfs_mime_get_magic_table (void)
 		/* don't have a pre-parsed table, use original text file */
 
 	        filename = g_strconcat (GNOME_VFS_CONFDIR, "/gnome-vfs-mime-magic", NULL);
+		mime_magic_table = gnome_vfs_mime_magic_parse (filename, NULL);
 		g_free (filename);
   	}
 
@@ -588,7 +617,7 @@ gnome_vfs_mime_test_get_magic_table (const char *table_path)
 {
 	G_LOCK (mime_magic_table_mutex);
   	if (mime_magic_table == NULL) {
-		mime_magic_table = gnome_vfs_mime_magic_parse(table_path, NULL);
+		mime_magic_table = gnome_vfs_mime_magic_parse (table_path, NULL);
   	}
 	G_UNLOCK (mime_magic_table_mutex);
 
@@ -747,6 +776,10 @@ gnome_vfs_sniff_buffer_looks_like_text (GnomeVFSMimeSniffBuffer *sniff_buffer)
 	
 	gnome_vfs_mime_sniff_buffer_get (sniff_buffer, GNOME_VFS_TEXT_SNIFF_LENGTH);
 
+	if (sniff_buffer->buffer_length == 0) {
+		return FALSE;
+	}
+	
 	for (index = 0; index < sniff_buffer->buffer_length - 3; index++) {
 		ch = sniff_buffer->buffer[index];
 		if (!isprint (ch) && !isspace(ch)) {
@@ -772,14 +805,14 @@ gnome_vfs_sniff_buffer_looks_like_text (GnomeVFSMimeSniffBuffer *sniff_buffer)
 			} else if ((ch & 0x30) == 0x20) {
 				/* check if this is a 3-byte UTF-8 letter */
 				if ((sniff_buffer->buffer[++index] & 0xc0) != 0x80
-					|| (sniff_buffer->buffer[++index] & 0xc0) != 0x80) {
+				    || (sniff_buffer->buffer[++index] & 0xc0) != 0x80) {
 					return FALSE;
 				}
 			} else if ((ch & 0x38) == 0x30) {
 				/* check if this is a 4-byte UTF-8 letter */
 				if ((sniff_buffer->buffer[++index] & 0xc0) != 0x80
-					|| (sniff_buffer->buffer[++index] & 0xc0) != 0x80
-					|| (sniff_buffer->buffer[++index] & 0xc0) != 0x80) {
+				    || (sniff_buffer->buffer[++index] & 0xc0) != 0x80
+				    || (sniff_buffer->buffer[++index] & 0xc0) != 0x80) {
 					return FALSE;
 				}
 			}
