@@ -25,7 +25,7 @@
 
 #include "gnome-vfs-module.h"
 
-typedef struct _FtpConnection {
+typedef struct {
 	GnomeVFSMethodHandle method_handle;
 	GnomeVFSInetConnection *inet_connection;
 	GnomeVFSIOBuf *iobuf;
@@ -44,6 +44,7 @@ typedef struct _FtpConnection {
 	} operation;
 	gchar *dirlist;
 	gchar *dirlistptr;
+	GnomeVFSFileInfoOptions file_info_options;
 } FtpConnection;
 
 #endif /* FTP_METHOD_H */
