@@ -1,3 +1,25 @@
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
+/* translate-method.c - translator method for GNOME Virtual File System
+
+   Copyright (C) 1999, 2000 Red Hat Inc.
+
+   The Gnome Library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public License as
+   published by the Free Software Foundation; either version 2 of the
+   License, or (at your option) any later version.
+
+   The Gnome Library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Library General Public License for more details.
+
+   You should have received a copy of the GNU Library General Public
+   License along with the Gnome Library; see the file COPYING.LIB.  If not,
+   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+
+   Author: Elliot Lee <sopwith@redhat.com> */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -59,7 +81,8 @@ tr_apply_default_mime_type(TranslateMethod * tm,
 	/* Only apply the default mime-type if the real method returns
 	 * unknown mime-type and the the TranslateMethod /has/ a mime-type */
 	if (file_info->mime_type == NULL) {
-		/* FIXME: this may not be needed since methods need to
+		/* FIXME bugzilla.eazel.com 2799: 
+		 * this may not be needed since methods need to
 		 * return application/octet-stream if it is an unknown
 		 * mime-type, not NULL */
 		if (tm->pa.default_mime_type) {
@@ -151,8 +174,7 @@ static pid_t tr_exec_open_child (char **argv, /*OUT*/ FILE ** p_from_stream, /*O
 
 	ret = gnome_vfs_forkexec (	argv[0],
 					(const char * const*)argv,
-					GNOME_VFS_PROCESS_SETSID 
-						| GNOME_VFS_PROCESS_CLOSEFDS,
+					GNOME_VFS_PROCESS_CLOSEFDS,
 					tr_forkexec_cb,
 					(gpointer) &cb_data
 	);
@@ -242,22 +264,31 @@ static void tr_exec_pass_uri (char *uri_string, FILE * out_stream)
 static char * tr_exec_do_retain (TranslateMethod *tm, char * uri_string)
 {
 	char * child_result = NULL;
-	
+	int tries;
+
 	g_mutex_lock (tm->exec_state.retain_lock);
 
-	if ( 0 == tm->exec_state.retain_pid ) {
-		tm->exec_state.retain_pid = tr_exec_open_child (tm->pa.u.exec.argv, &(tm->exec_state.retain_from), &(tm->exec_state.retain_to));
-		if ( -1 == tm->exec_state.retain_pid ) {
+
+	/* re-execute the child process in case it unexpectedly disappears */  
+	for (tries = 0 ; ( ! child_result ) && tries < 3 ; tries ++) {
+		if ( 0 == tm->exec_state.retain_pid ) {
+			tm->exec_state.retain_pid = tr_exec_open_child (tm->pa.u.exec.argv, &(tm->exec_state.retain_from), &(tm->exec_state.retain_to));
+			if ( -1 == tm->exec_state.retain_pid ) {
+				tm->exec_state.retain_pid = 0;
+				goto error;
+			}
+		}
+
+		g_assert (uri_string);
+		tr_exec_pass_uri (uri_string, tm->exec_state.retain_to);
+
+		child_result = tr_getline (tm->exec_state.retain_from);
+
+		if (NULL == child_result) {
 			tm->exec_state.retain_pid = 0;
-			goto error;
 		}
 	}
-
-	g_assert (uri_string);
-	tr_exec_pass_uri (uri_string, tm->exec_state.retain_to);
-
-	child_result = tr_getline (tm->exec_state.retain_from);
-
+	
 error:
 	g_mutex_unlock (tm->exec_state.retain_lock);
 
@@ -300,7 +331,7 @@ error:
  * (note that sed won't work with -retain)
  */
 
-/* FIXME: this may be broken when the child produces compound URI's */
+/* FIXME bugzilla.eazel.com 2796: this may be broken when the child produces compound URI's */
 static GnomeVFSURI *tr_handle_exec (TranslateMethod *tm, const GnomeVFSURI * uri)
 {
 
@@ -328,6 +359,10 @@ static GnomeVFSURI *tr_handle_exec (TranslateMethod *tm, const GnomeVFSURI * uri
 
 	if (tm->pa.u.exec.retain) {
 		child_result = tr_exec_do_retain (tm, uri_string);
+
+		if ( NULL == child_result ) {
+			goto error;
+		}
 	} else {
 		pid_t child_pid;
 		FILE *from_stream, *to_stream;
@@ -362,7 +397,8 @@ static GnomeVFSURI *tr_handle_exec (TranslateMethod *tm, const GnomeVFSURI * uri
 	}
 
 
-	/* FIXME: just because we've appended the same scheme, doesn't mean
+	/* FIXME bugzilla.eazel.com 2797: 
+	 * just because we've appended the same scheme, doesn't mean
 	 * we're going to get the same method from gnome_vfs_uri_new
 	 */
 
@@ -422,6 +458,8 @@ static GnomeVFSURI *tr_uri_translate(TranslateMethod * tm,
 {
 	GnomeVFSURI *retval;
 
+	retval = NULL;
+
 	if (uri->method != (GnomeVFSMethod *) tm)
 		return gnome_vfs_uri_ref((GnomeVFSURI *) uri);	/* Don't translate things that don't belong to us */
 
@@ -443,8 +481,6 @@ static GnomeVFSURI *tr_uri_translate(TranslateMethod * tm,
 		g_assert (FALSE);
 	}
 
-	/* FIXME callers need to be aware that this can now return NULL */
-
 	return retval;
 }
 
@@ -750,6 +786,7 @@ tr_do_find_directory(GnomeVFSMethod * method,
 		     GnomeVFSFindDirectoryKind kind,
 		     GnomeVFSURI ** result_uri,
 		     gboolean create_if_needed,
+		     gboolean find_if_needed,
 		     guint permissions, GnomeVFSContext * context)
 {
 	TranslateMethod *tm = (TranslateMethod *) method;
@@ -762,7 +799,8 @@ tr_do_find_directory(GnomeVFSMethod * method,
 		retval =
 		    tm->real_method->find_directory(tm->real_method, real_uri,
 						    kind, result_uri,
-						    create_if_needed, permissions,
+						    create_if_needed, find_if_needed,
+						    permissions,
 						    context);
 
 		gnome_vfs_uri_unref(real_uri);
